from logging import getLogger
from pathlib import Path

from pandas import DataFrame
from requests_cache import CachedSession

LOG = getLogger(__name__)

CACHE_PATH = Path.home() / ".cache" / "geekbench_browser" / "cache.sqlite"
GEEKBENCH_URL = "https://browser.geekbench.com/processor-benchmarks.json"


def get_data() -> DataFrame:
    with CachedSession(
        CACHE_PATH.as_posix(), backend="sqlite", expire_after=86400
    ) as session:
        resp = session.get(GEEKBENCH_URL)
        LOG.info(
            f"Last retrieved: {resp.created_at}, Cache expires: {resp.expires}, "
            f"Cache path: {CACHE_PATH}"
        )

        json_data = session.get(GEEKBENCH_URL).json()
        try:
            df = DataFrame(json_data["devices"])
        except Exception as e:
            raise RuntimeError("Failed to parse json data") from e
        df.set_index("name", inplace=True, drop=False)
        df.index.name = None
        df["frequency"] = (
            df["description"]
            .str.extract(r"(\d+\.?\d+) [MG]Hz")
            .astype(float, errors="ignore")
        )
        df.loc[df["description"].str.contains("MHz"), "frequency"] *= 1e-3
        df["cores"] = (
            df["description"].str.extract(r"(\d+) cores").astype(int, errors="ignore")
        )
        df = df.rename(columns={"score": "single", "multicore_score": "multi"})
        return df



from typing import Optional

import pandas as pd
from rich.table import Table


def df_to_table(
    pandas_dataframe: pd.DataFrame,
    rich_table: Table,
    show_index: bool = True,
    index_name: Optional[str] = None,
) -> Table:
    """Convert a pandas.DataFrame obj into a rich.Table obj.

    Args:
        pandas_dataframe (DataFrame):
            A Pandas DataFrame to be converted to a rich Table.
        rich_table (Table):
            A rich Table that should be populated by the DataFrame values.
        show_index (bool):
            Add a column with a row count to the table. Defaults to True.
        index_name (str, optional):
            The column name to give to the index column.
            Defaults to None, showing no value.

    Returns:
        Table:
            The rich Table instance passed, populated with the DataFrame values."""

    if show_index:
        index_name = str(index_name) if index_name else ""
        rich_table.add_column(index_name)
        rich_indexes = pandas_dataframe.index.to_list()

    for column in pandas_dataframe.columns:
        rich_table.add_column(str(column))

    for index, value_list in enumerate(pandas_dataframe.values.tolist()):
        row = [str(rich_indexes[index])] if show_index else []
        row += [str(x) for x in value_list]
        rich_table.add_row(*row)

    return rich_table



from typing import Any

import click
from click import Context, OptionParser
from click.parser import ParsingState


class OptionEatAll(click.Option):
    def __init__(self, *args: Any, **kwargs: Any):
        self.save_other_options = kwargs.pop("save_other_options", True)
        nargs = kwargs.pop("nargs", -1)
        if nargs != -1:
            raise ValueError(f"nargs, if set, must be -1 not {nargs}")
        super().__init__(*args, **kwargs)

    def add_to_parser(self, parser: OptionParser, ctx: Context) -> None:
        def parser_process(value: str, state: ParsingState) -> None:
            # method to hook to the parser.process
            done = False
            values = [value]
            if self.save_other_options:
                # grab everything up to the next option
                while state.rargs and not done:
                    for prefix in self._eat_all_parser.prefixes:
                        if state.rargs[0].startswith(prefix):
                            done = True
                    if not done:
                        values.append(state.rargs.pop(0))
            else:
                # grab everything remaining
                values += state.rargs
                state.rargs[:] = []

            # call the actual process
            self._previous_parser_process(tuple(values), state)

        retval = super().add_to_parser(parser, ctx)
        for name in self.opts:
            our_parser = parser._long_opt.get(name) or parser._short_opt.get(name)
            if our_parser:
                self._eat_all_parser = our_parser
                self._previous_parser_process = our_parser.process
                our_parser.process = parser_process
                break
        return retval



from logging import INFO, basicConfig, getLogger

import click

from ._main import get_data
from ._option_eat_all import OptionEatAll

LOG = getLogger(__name__)


# When using OptionEatAll, type must be tuple
@click.help_option("--help", "-h")
@click.command()
@click.argument("names", nargs=-1, type=str)
@click.option("--min-cores", "-mc", type=int, help="Minimum number of cores")
@click.option("--max-cores", "-xc", type=int, help="Maximum number of cores")
@click.option("--min-frequency", "-mf", type=float, help="Minimum frequency in GHz")
@click.option("--max-frequency", "-xf", type=float, help="Maximum frequency in GHz")
@click.option("--min-single", "-ms", type=float, help="Minimum single core score")
@click.option("--max-single", "-xs", type=float, help="Maximum single core score")
@click.option("--min-multi", "-mm", type=float, help="Minimum multi core score")
@click.option("--max-multi", "-xm", type=float, help="Maximum multi core score")
@click.option("--icon", "-i", cls=OptionEatAll, type=tuple, help="Icon to search for")
@click.option(
    "--family", "-f", cls=OptionEatAll, type=tuple, help="Family to search for"
)
@click.option(
    "--sort",
    "-s",
    type=click.Choice(["name", "single", "multi", "frequency", "cores", "id"]),
    default="name",
    help="Sort by (reverse is default except for name, id)",
)
@click.option("--reverse", "-r", is_flag=True, help="Reverse the sort order")
@click.option("--verbose", "-v", is_flag=True, help="Verbose output")
def cli(
    names: list[str],
    min_cores: int,
    max_cores: int,
    min_frequency: float,
    max_frequency: float,
    min_single: float,
    max_single: float,
    min_multi: float,
    max_multi: float,
    icon: list[str],
    family: list[str],
    sort: str,
    reverse: bool,
    verbose: bool,
) -> None:
    if verbose:
        basicConfig(level=INFO)

    # get data
    data = get_data()
    if "current" in names:
        import cpuinfo

        # get current cpu name
        name = cpuinfo.get_cpu_info()["brand_raw"]
        LOG.info(f"Current CPU: {name}")

        # geekbench cpuname is shorter than cpuinfo brand_raw
        current_names = data.loc[data["name"].map(lambda x: x in name)]["name"]
        if current_names.empty:
            raise ValueError(f"Could not find current CPU name in database: {name}")

        # remove current from names and add current_names
        names = [n for n in names if n != "current"] + list(current_names)

    # filter data
    if names:
        data = data.loc[
            data["name"].str.contains("|".join(names), regex=True, case=False)
        ]
    if min_cores:
        data = data.loc[data["cores"] >= min_cores]
    if max_cores:
        data = data.loc[data["cores"] <= max_cores]
    if min_frequency:
        data = data.loc[data["frequency"] >= min_frequency]
    if max_frequency:
        data = data.loc[data["frequency"] <= max_frequency]
    if min_single:
        data = data.loc[data["single"] >= min_single]
    if max_single:
        data = data.loc[data["single"] <= max_single]
    if min_multi:
        data = data.loc[data["multi"] >= min_multi]
    if max_multi:
        data = data.loc[data["multi"] <= max_multi]
    if icon:
        data = data.loc[
            data["icon"]
            .astype(str)
            .str.contains("|".join(icon), regex=True, case=False)
        ]
    if family:
        data = data.loc[
            data["family"]
            .astype(str)
            .str.contains("|".join(family), regex=True, case=False)
        ]

    # sort data
    if sort in ["name", "id"]:
        reverse = not reverse
    data.sort_values(by=sort, inplace=True, ascending=reverse)

    # drop columns that does not make sense
    if not verbose:
        data = data.drop(columns=["cores", "frequency", "id", "name"])
    data = data.reindex(
        columns=["description", "single", "multi", "icon", "family", "samples"]
    )

    USE_RICH = True
    if USE_RICH:
        import rich
        import rich.box
        import rich.table

        from ._rich_table import df_to_table

        console = rich.console.Console()
        table = rich.table.Table()
        console.print(df_to_table(data, table))
    else:
        click.echo(data)


cli()
